@@ -1,6 +1,5 @@
 import os
 import ssl
-import sys
 import tempfile
 import textwrap
 from pathlib import Path
@@ -260,10 +259,6 @@ def test_install_no_binary_via_config_disables_cached_wheels(
     assert "Building wheel for upper" in str(res), str(res)
 
 
-@pytest.mark.skipif(
-    sys.platform == "linux" and sys.version_info < (3, 8),
-    reason="Custom SSL certification not running well in CI",
-)
 def test_prompt_for_authentication(
     script: PipTestEnvironment, data: TestData, cert_factory: CertFactory
 ) -> None:
@@ -304,10 +299,6 @@ def test_prompt_for_authentication(
     assert f"User for {server.host}:{server.port}" in result.stdout, str(result)
 
 
-@pytest.mark.skipif(
-    sys.platform == "linux" and sys.version_info < (3, 8),
-    reason="Custom SSL certification not running well in CI",
-)
 def test_do_not_prompt_for_authentication(
     script: PipTestEnvironment, data: TestData, cert_factory: CertFactory
 ) -> None:
@@ -398,10 +389,6 @@ def flags(
     return flags
 
 
-@pytest.mark.skipif(
-    sys.platform == "linux" and sys.version_info < (3, 8),
-    reason="Custom SSL certification not running well in CI",
-)
 def test_prompt_for_keyring_if_needed(
     data: TestData,
     cert_factory: CertFactory,
