@@ -5,9 +5,9 @@ from pathlib import Path
 import pytest
 
 from pip._internal.models.direct_url import DirectUrl, DirInfo
-from tests.conftest import ScriptFactory
 from tests.lib import (
     PipTestEnvironment,
+    ScriptFactory,
     TestData,
     _create_test_package,
     create_test_package_with_setup,
@@ -273,25 +273,19 @@ def test_outdated_flag(script: PipTestEnvironment, data: TestData) -> None:
         "latest_version": "3.0",
         "latest_filetype": "sdist",
     } in json_output
-    assert (
-        dict(
-            name="simplewheel",
-            version="1.0",
-            latest_version="2.0",
-            latest_filetype="wheel",
-        )
-        in json_output
-    )
-    assert (
-        dict(
-            name="pip-test-package",
-            version="0.1",
-            latest_version="0.1.1",
-            latest_filetype="sdist",
-            editable_project_location="<location>",
-        )
-        in json_output
-    )
+    assert {
+        "name": "simplewheel",
+        "version": "1.0",
+        "latest_version": "2.0",
+        "latest_filetype": "wheel",
+    } in json_output
+    assert {
+        "name": "pip-test-package",
+        "version": "0.1",
+        "latest_version": "0.1.1",
+        "latest_filetype": "sdist",
+        "editable_project_location": "<location>",
+    } in json_output
     assert "simple2" not in {p["name"] for p in json_output}
 
 
@@ -601,8 +595,7 @@ def test_outdated_formats(script: PipTestEnvironment, data: TestData) -> None:
         "--outdated",
         "--format=json",
     )
-    data = json.loads(result.stdout)
-    assert data == [
+    assert json.loads(result.stdout) == [
         {
             "name": "simple",
             "version": "1.0",
