@@ -1,4 +1,5 @@
 """'pip wheel' tests"""
+
 import os
 import re
 import sys
@@ -10,7 +11,7 @@ from pip._internal.cli.status_codes import ERROR
 from tests.lib import (
     PipTestEnvironment,
     TestData,
-    pyversion,  # noqa: F401
+    pyversion,
 )
 
 
@@ -59,9 +60,7 @@ def test_pip_wheel_success(script: PipTestEnvironment, data: TestData) -> None:
     wheel_file_path = script.scratch / wheel_file_name
     assert re.search(
         r"Created wheel for simple: "
-        r"filename={filename} size=\d+ sha256=[A-Fa-f0-9]{{64}}".format(
-            filename=re.escape(wheel_file_name)
-        ),
+        rf"filename={re.escape(wheel_file_name)} size=\d+ sha256=[A-Fa-f0-9]{{64}}",
         result.stdout,
     )
     assert re.search(r"^\s+Stored in directory: ", result.stdout, re.M)
