@@ -28,10 +28,7 @@ def mock_permitted(ups: UninstallPathSet, path: str) -> bool:
 def test_uninstallation_paths() -> None:
     class dist:
         def iter_declared_entries(self) -> Optional[Iterator[str]]:
-            yield "file.py"
-            yield "file.pyc"
-            yield "file.so"
-            yield "nopyc.py"
+            return iter(["file.py", "file.pyc", "file.so", "nopyc.py"])
 
         location = ""
 
@@ -59,10 +56,9 @@ def test_uninstallation_paths() -> None:
 
 def test_compressed_listing(tmpdir: Path) -> None:
     def in_tmpdir(paths: List[str]) -> List[str]:
-        li = []
-        for path in paths:
-            li.append(str(os.path.join(tmpdir, path.replace("/", os.path.sep))))
-        return li
+        return [
+            str(os.path.join(tmpdir, path.replace("/", os.path.sep))) for path in paths
+        ]
 
     sample = in_tmpdir(
         [
